# -*- coding: utf8 -*-
from __future__ import unicode_literals

import csv
import datetime
import logging

import six
from six import BytesIO, StringIO

from driverng import MessageV2, Worklist
from driverng.exceptions import ParametrageError
from driverng.splitted import Splitted, String, Multiple

logger = logging.getLogger("driverng.quantstudio_interpretive")


class QuantStudioBase(Splitted):

    SEPARATOR = ","
    ENCODING = "utf-8"


class QuantStudioResult(object):
    FIELDS = [
        String("batch_id"),
        String("sample_name"),
        String("well"),
        String("well_number"),
        String("sample_type"),
        Multiple("results", (String("resu"),), separator=","),
    ]

    def parse(self, line):
        fields = self.FIELDS
        for i, field in enumerate(fields):
            if isinstance(field, Multiple):
                setattr(self, field.name, line[i:])
                break
            else:
                setattr(self, field.name, line[i])


class QuantStudioResultsMultiple(list):
    _MESSAGE_TYPE_MAP = {MessageV2.RESULT: QuantStudioResult}

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        if six.PY2:
            buf = BytesIO(data.encode("utf-8"))
        else:
            buf = StringIO(data)
        reader = csv.reader(buf, delimiter=str(","))
        self.date_end_result = None
        self.instrument_serial_number = None
        self.operator = None
        for row in reader:
            if not row or not len(row):
                continue
            if six.PY2:
                line = [
                    c.decode("utf-8") for c in row
                ]  # PY2 csv reader est imcompatible avec unicode
            else:
                line = row
            if "Run End Date" in line[0]:
                try:
                    self.date_end_result = datetime.datetime.strptime(
                        line[0].split("=")[1].strip(), "%a %b %d %H:%M:%S %Z %Y"
                    )
                except ValueError as e:
                    logger.info(
                        "La date des résultats (Run End Date) n'est pas format attendu %s",
                        e,
                    )
            if "Instrument Serial Number" in line[0]:
                self.instrument_serial_number = line[0].split("=")[1].strip()
            if "Run Operator" in line[0]:
                self.operator = line[0].split("=")[1].strip()
            if not len(line):
                continue
            if len(line) > 5:
                obj = self.add(type_message=MessageV2.RESULT)
                obj.parse(line)


class Driver(MessageV2):
    SPEC = {"code": "quantstudio", "label": "QuantStudio Interpretive QS5 "}
    OPTIONS = [
        MessageV2.Option("analyseur_traites", label="Liste des analyseurs traités")
    ]

    def __init__(self, **kwargs):
        analyseur_traites = kwargs.pop("analyseur_traites", None)
        self.analyseur_traites = (
            analyseur_traites.split(",") if analyseur_traites is not None else None
        )
        super(Driver, self).__init__(**kwargs)

    def parse(self, data):
        obj = QuantStudioResultsMultiple()
        obj.parse(data)
        return obj

    def decode_message(self, message):
        worklist = Worklist(context="result")
        RESULTS_CODES = ()
        if (
            self.analyseur_traites is not None
            and message.instrument_serial_number not in self.analyseur_traites
        ):
            raise ParametrageError(
                "l'instrument %s n'est pas traité par la connexion"
                % message.instrument_serial_number
            )
        for m in message:
            if (
                m.batch_id == "Batch ID"
                and m.sample_name == "Sample Name"
                and m.well == "Well"
            ):
                RESULTS_CODES = m.results
                continue
            if RESULTS_CODES:
                sample = worklist.get_or_create(
                    sample_id=m.sample_name.strip(), rack=m.well, type="patient"
                )
                for index, result in enumerate(m.results):
                    if RESULTS_CODES[index] and result:
                        sample.results.add(
                            code=RESULTS_CODES[index].strip(),
                            value=result.strip(),
                            date=message.date_end_result,
                            operator=message.operator,
                            instrument=message.instrument_serial_number,
                        )
        return worklist
